unit MainUnit;

interface

uses
    Forms, StdCtrls, ComCtrls, Controls, Classes;
type
  TMainForm = class(TForm)
    TypeLabel: TLabel;
    TypeComboBox: TComboBox;
    ExtAsciiCheckBox: TCheckBox;
    ParentLabel: TLabel;
    ParentEdit: TEdit;
    Sublabel: TLabel;
    SubEdit: TEdit;
    WorkButton: TButton;
    Memo: TRichEdit;
    procedure FormShow(Sender: TObject);
    procedure WorkButtonClick(Sender: TObject);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
  private
    Procedure Info(const Line : String; const Heading : Boolean = False);
  public
  end;

var
  MainForm: TMainForm;

implementation

uses Tools, FileCtrl, FileNameConvertor, SysUtils, Graphics, Messages, Windows;

{$R *.dfm}

const CSIDL_PROFILE = $0028;

procedure TMainForm.FormShow(Sender: TObject);
Var S : String;
begin
  TypeComboBox.ItemIndex:=0;
  ExtAsciiCheckBox.Checked:=False;
  ActiveControl:=ParentEdit;

  SubEdit.Text:='LongDirName';
  
  S:=IncludeTrailingBackslash(GetSpecialFolder(Handle,CSIDL_PROFILE));
  If DirectoryExists(S+'D-Fend Reloaded\VirtualHD') then begin ParentEdit.Text:=S+'D-Fend Reloaded\VirtualHD'; exit; end;
  If DirectoryExists(S+'D-Fend Reloaded') then begin ParentEdit.Text:=S+'D-Fend Reloaded'; exit; end;
  ParentEdit.Text:=S;
end;

procedure TMainForm.Info(const Line: String; const Heading: Boolean);
begin
  If Heading then begin
    If Memo.Lines.Count>0 then Memo.Lines.Add('');
    Memo.SelAttributes.Style:=[fsBold];
  end;
  Memo.Lines.Add(Line);
  Memo.SelAttributes.Style:=[];
end;

procedure TMainForm.EditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#13 then
     WorkButtonClick(Sender);
end;

procedure TMainForm.WorkButtonClick(Sender: TObject);
Var PDir, SDir, S : String;
    I : Integer;
    ConvertResult: TConvertResult;
    UseLinuxMode,UseSystemShortName, UseSystemFallback, UseExtendedAscii:Boolean;
    ShortName: String;
    isDir: Boolean;
    Dir: TFileNameConvertor;
    nInfo: TFileNameInfo;
begin
  Memo.Lines.Clear;

  If (Trim(ParentEdit.Text)='') or (Trim(SubEdit.Text)='') then exit;

  PDir:=IncludeTrailingBackslash(ParentEdit.Text);
  SDir:=SubEdit.Text;

  Info('Input data:',True);
  Info('Parent directory: '+PDir);
  Info('Subdirectory: '+SDir);

  UseLinuxMode:=TypeComboBox.ItemIndex>2;
  UseSystemShortName:=TypeComboBox.ItemIndex in [2,4];
  UseSystemFallback:=TypeComboBox.ItemIndex=0;
  UseExtendedAscii:=ExtAsciiCheckBox.Checked;

//Type TConvertResult = (tcrErrParentNotFound, tcrErrDelimiterInFileName, tcrErrFileNotFound,
//        tcrWarnInvalidChars, tcrWarnSystemShort, tcrWarnSystemShortAlreadyUsed,
//        tcrOkExternal, tcrOk);

  ConvertResult:=ConvertLongFileNameToShort(PDir,SDir,UseLinuxMode,UseSystemShortName,UseSystemFallback,UseExtendedAscii,ShortName,isDir);

  case ConvertResult of
    tcrErrParentNotFound, tcrErrDelimiterInFileName, tcrErrFileNotFound:
        Info('Error:',True);
    tcrWarnInvalidChars, tcrWarnSystemShort, tcrWarnSystemShortAlreadyUsed:
        Info('Warning:',True);
    tcrOkExternal, tcrOk:
        Info('Message:',True);
  end;
  case ConvertResult of
    tcrErrParentNotFound:
        Info('Parent directory "'+PDir+'" does not exist.');
    tcrErrDelimiterInFileName:
        Info('File/subdirectory name "'+SDir+'" contains path delimiters (\/:).');
    tcrErrFileNotFound:
        Info('File or subdirectory "'+SDir+'" does not exist in parent directory "'+PDir+'".');
    tcrWarnInvalidChars:
        Info('File/subdirectory name converted to DOSBox form, but it has illegal characters. Generated name may be not unavailable under DOSBox.');
    tcrWarnSystemShort:
        Info('File/subdirectory name converted to DOSBox form has illegal characters, hence system shortname will be used. Generated name may be not portable between different computers.');
    tcrWarnSystemShortAlreadyUsed:
        Info('File/subdirectory name converted to DOSBox form has illegal characters, but system shortname cannot be used, because it is already used for another file. Hence DOSBox short name is left, but it may be be not unavailable under DOSBox.');
    tcrOkExternal:
        Info('System shortest name was generated. All is OK.');
    tcrOk:
        Info('DOSBox short name was generated. All is OK.');
  end;

  S:=ShortName;
  if isDir then S:=S+' <DIR>';
  Info('Result:',True);
  Info(S);

  Dir:=TFileNameConvertor.Create(PDir,UseLinuxMode);
  try
    if Dir.ItemsCount>0 then
    begin
        Info('Scanned parent directory:',True);
        for I:=0 to Dir.ItemsCount-1 do
        begin
            nInfo:=Dir.Items[i];
            S:=nInfo.LongName;
            if(nInfo.isDir) then S:=S+' <DIR>';
            S:=S+' -> '+nInfo.ShortName;
            Info(s);
        end
    end
  finally
    Dir.Free();
  end;

  Memo.SelStart:=0;
  Memo.SelLength:=0;
  PostMessage(Memo.Handle,EM_SCROLLCARET,0,0);
end;

end.
